==> gromacs_2020.6.orig.tar.gz: gromacs-2020.6/src/gromacs/gmxpreprocess/gpp_bond_atomtype.cpp <==
/*
 * This file is part of the GROMACS molecular simulation package.
 *
 * Copyright (c) 1991-2000, University of Groningen, The Netherlands.
 * Copyright (c) 2001-2004, The GROMACS development team.
 * Copyright (c) 2011,2014,2015,2017,2018,2019, by the GROMACS development team, led by
 * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
 * and including many others, as listed in the AUTHORS file in the
 * top-level source directory and at http://www.gromacs.org.
 *
 * GROMACS is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * GROMACS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with GROMACS; if not, see
 * http://www.gnu.org/licenses, or write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * If you want to redistribute modifications to GROMACS, please
 * consider that scientific software is very special. Version
 * control is crucial - bugs must be traceable. We will be happy to
 * consider code for inclusion in the official distribution, but
 * derived work must not be called official GROMACS. Details are found
 * in the README & COPYING files - if they are missing, get the
 * official version at http://www.gromacs.org.
 *
 * To help us fund GROMACS development, we humbly ask that you cite
 * the research papers on the package. Check out http://www.gromacs.org.
 */
#include "gmxpre.h"

#include "gpp_bond_atomtype.h"

#include <cstring>

#include <algorithm>
#include <vector>

#include "gromacs/gmxpreprocess/notset.h"
#include "gromacs/topology/symtab.h"
#include "gromacs/utility/cstringutil.h"
#include "gromacs/utility/smalloc.h"

class PreprocessingBondAtomType::Impl
{
public:
    //! The atom type names.
    std::vector<char**> typeNames;
};

int PreprocessingBondAtomType::bondAtomTypeFromName(const std::string& str) const
{
    /* Atom types are always case sensitive */
    auto found =
            std::find_if(impl_->typeNames.begin(), impl_->typeNames.end(),
                         [&str](const auto& type) { return str == const_cast<const char*>(*type); });
    if (found == impl_->typeNames.end())
    {
        return NOTSET;
    }
    else
    {
        return std::distance(impl_->typeNames.begin(), found);
    }
}

const char* PreprocessingBondAtomType::atomNameFromBondAtomType(int nt) const
{
    return isSet(nt) ? *impl_->typeNames[nt] : nullptr;
}

PreprocessingBondAtomType::PreprocessingBondAtomType() : impl_(new Impl) {}

PreprocessingBondAtomType::~PreprocessingBondAtomType() {}

int PreprocessingBondAtomType::addBondAtomType(t_symtab* tab, const std::string& name)
{
    int position = bondAtomTypeFromName(name);
    if (position == NOTSET)
    {
        impl_->typeNames.emplace_back(put_symtab(tab, name.c_str()));
        return bondAtomTypeFromName(name);
    }
    else
    {
        return position;
    }
}

size_t PreprocessingBondAtomType::size() const
{
    return impl_->typeNames.size();
}

bool PreprocessingBondAtomType::isSet(int nt) const
{
    return ((nt >= 0) && (nt < gmx::ssize(*this)));
}

==> gromacs-2022.6.tar.gz: gromacs-2022.6/src/gromacs/gmxpreprocess/gpp_bond_atomtype.cpp <==
/*
 * This file is part of the GROMACS molecular simulation package.
 *
 * Copyright 1991- The GROMACS Authors
 * and the project initiators Erik Lindahl, Berk Hess and David van der Spoel.
 * Consult the AUTHORS/COPYING files and https://www.gromacs.org for details.
 *
 * GROMACS is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * GROMACS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with GROMACS; if not, see
 * https://www.gnu.org/licenses, or write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * If you want to redistribute modifications to GROMACS, please
 * consider that scientific software is very special. Version
 * control is crucial - bugs must be traceable. We will be happy to
 * consider code for inclusion in the official distribution, but
 * derived work must not be called official GROMACS. Details are found
 * in the README & COPYING files - if they are missing, get the
 * official version at https://www.gromacs.org.
 *
 * To help us fund GROMACS development, we humbly ask that you cite
 * the research papers on the package. Check out https://www.gromacs.org.
 */
#include "gmxpre.h"

#include "gpp_bond_atomtype.h"

#include <cstring>

#include <algorithm>
#include <optional>
#include <vector>

#include "gromacs/gmxpreprocess/notset.h"
#include "gromacs/topology/symtab.h"
#include "gromacs/utility/smalloc.h"

class PreprocessingBondAtomType::Impl
{
public:
    //! The atom type names.
    std::vector<char**> typeNames;
};

std::optional<int> PreprocessingBondAtomType::bondAtomTypeFromName(const std::string& str) const
{
    /* Atom types are always case sensitive */
    auto found = std::find_if(impl_->typeNames.begin(),
                              impl_->typeNames.end(),
                              [&str](const auto& type) { return str == std::string(*type); });
    if (found == impl_->typeNames.end())
    {
        return std::nullopt;
    }
    else
    {
        return std::make_optional(std::distance(impl_->typeNames.begin(), found));
    }
}

std::optional<const char*> PreprocessingBondAtomType::atomNameFromBondAtomType(int nt) const
{
    return isSet(nt) ? *impl_->typeNames[nt] : std::optional<const char*>{};
}

PreprocessingBondAtomType::PreprocessingBondAtomType() : impl_(new Impl) {}

PreprocessingBondAtomType::~PreprocessingBondAtomType() {}

int PreprocessingBondAtomType::addBondAtomType(t_symtab* tab, const std::string& name)
{
    auto position = bondAtomTypeFromName(name);
    if (!position.has_value())
    {
        impl_->typeNames.emplace_back(put_symtab(tab, name.c_str()));
        if (auto bondAtomType = bondAtomTypeFromName(name); bondAtomType.has_value())
        {
            return *bondAtomType;
        }
        else
        {
            GMX_RELEASE_ASSERT(false, "Unhandled error in adding bond atom type");
            return 0;
        }
    }
    else
    {
        return *position;
    }
}

size_t PreprocessingBondAtomType::size() const
{
    return impl_->typeNames.size();
}

bool PreprocessingBondAtomType::isSet(int nt) const
{
    return ((nt >= 0) && (nt < gmx::ssize(*this)));
}
